/*
 *  ncxmms
 *  Copyright 2003 Mariusz Gniazdowski
 *
 *  NCXmms is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  NCXmms is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <ctype.h>
#define __USE_GNU 1
#include <string.h>
#undef __USE_GNU
#include <ncurses.h>
#include <menu.h>
#include <xmms/xmmsctrl.h>
#include "defines.h"
#include "file.h"
#include "playlist.h"
#include "status.h"


/*
 * NAME:        main_control
 * DESCRIPTION: Main input loop. It reads keys and calls appropriate 
 * 		functions to handle requests.
 *
 * 		Pattern jump is O(n^2), but it works...
 * 		 
 */

int main_control(struct dir_data_s *dir_data, 
		WINDOW *wintab[], MENU *menutab[], ITEM **itemtab[], 
		int *actual, struct plist_data_s *plist_data, 
		int menu_width, int menu_height)
{
	int character, i, j, idx, tmp;
	int vl, vr, pattern_jump_mode=false;
	char buff[50];

	buff[0]='\0';

	/* wait 0.2sec for key, then display time */
	halfdelay(2);

	while (1) {
		character = wgetch(wintab[*actual]);
		if (character == ERR) {
			j = xmms_remote_get_output_time(0);
			mvwprintw(wintab[AREA_STAT], 0, 4, "%.2d:%.2d", j / 60000, (j - (j / 60000) * 60000) / 1000);
			wrefresh(wintab[AREA_STAT]);
			continue;
		}
		cbreak();

		if(pattern_jump_mode)
		{
			switch (character) 
			{
				case KEY_DOWN:
					if(plist_data->size<=1)
						break;
					/* Walk thru playlist till match is found
					 * (there must be at least one match, but
					 * this code is protected against that to be sure */
					j = 1;
					i = item_index(current_item(menutab[AREA_PLIST]));
					do {
						if(i==plist_data->size-1)
							menu_driver(menutab[AREA_PLIST],REQ_FIRST_ITEM);
						else
							menu_driver(menutab[AREA_PLIST],REQ_DOWN_ITEM);
						i = item_index(current_item(menutab[AREA_PLIST]));
						/* No match in whole plist ? */
						if(j==plist_data->size)
							break;
						j++;
					} while(NULL==strcasestr(plist_data->entries[i],buff));
					wrefresh(wintab[AREA_PLIST]);
					break;
				case KEY_UP:
					if(plist_data->size<=1)
						break;
					j = 1;
					i = item_index(current_item(menutab[AREA_PLIST]));
					do {
						if(i==0)
							menu_driver(menutab[AREA_PLIST],REQ_LAST_ITEM);
						else
							menu_driver(menutab[AREA_PLIST],REQ_UP_ITEM);
						i = item_index(current_item(menutab[AREA_PLIST]));
						if(j==plist_data->size)
							break;
						j++;
					} while(NULL==strcasestr(plist_data->entries[i],buff));
					wrefresh(wintab[AREA_PLIST]);
					break;
				case '\t':
					wattrset(wintab[WIN_STAT], COLOR_PAIR(HIGHLIGHT) | A_BOLD );
					wprintw(wintab[WIN_STAT],"\n----- Normal mode -----");
					wattrset(wintab[WIN_STAT], COLOR_PAIR(NORMAL) );
					wrefresh(wintab[WIN_STAT]);
					pattern_jump_mode=false;
					break;
				case '\n':
					xmms_remote_set_playlist_pos(0, item_index(current_item(menutab[AREA_PLIST])));
					wattrset(wintab[WIN_STAT], COLOR_PAIR(HIGHLIGHT) | A_BOLD );
					wprintw(wintab[WIN_STAT],"\n----- Normal mode -----");
					wattrset(wintab[WIN_STAT], COLOR_PAIR(NORMAL) );
					wrefresh(wintab[WIN_STAT]);
					pattern_jump_mode=false;
					break;
				case KEY_BACKSPACE:
					if(idx>0) {
						idx--;
						buff[idx]='\0';
						waddch(wintab[WIN_STAT],'\b');
						waddch(wintab[WIN_STAT],' ');
						waddch(wintab[WIN_STAT],'\b');
						wrefresh(wintab[WIN_STAT]);
					}
					break;
				default:
					if(isprint(character) && plist_data->size>0 && idx<38)
					{
						/* Add char to buffer */
						buff[idx++]=(char)character;
						buff[idx]='\0';
						waddch(wintab[WIN_STAT],character);
						wrefresh(wintab[WIN_STAT]);
						/* Walk thru playlist till match is found */
						j = 0;
						i = item_index(current_item(menutab[AREA_PLIST]));
						while(NULL==strcasestr(plist_data->entries[i],buff))
						{
							j++;
							/* No match in whole plist ? */
							if(j==plist_data->size)
							{
								wprintw(wintab[WIN_STAT],"\nNo match, removing last character...");
								if(idx>0) {
									idx--;
									buff[idx]='\0';
								}
								j=0;
								wprintw(wintab[WIN_STAT],"\nPattern: %s",buff);
								wrefresh(wintab[WIN_STAT]);
							}								
							/* End of menu -> jump to begining */
							if(i==plist_data->size-1)
								menu_driver(menutab[AREA_PLIST],REQ_FIRST_ITEM);
							else
								/* If no end, make step forward */
								menu_driver(menutab[AREA_PLIST],REQ_DOWN_ITEM);
							i = item_index(current_item(menutab[AREA_PLIST]));
						}
						wrefresh(wintab[AREA_PLIST]);
					}
					break;
			}

		} else {

			switch (character) 
			{
				case 'q':
					return 1;
				case KEY_DOWN:
					menu_driver(menutab[*actual], REQ_DOWN_ITEM);
					break;
				case KEY_UP:
					menu_driver(menutab[*actual], REQ_UP_ITEM);
					break;
				case KEY_LEFT:
					i = xmms_remote_get_output_time(0) - 5000;
					if (i < 0)
						i = 0;
					xmms_remote_jump_to_time(0, i);
					break;
				case KEY_RIGHT:
					i = xmms_remote_get_output_time(0) + 5000;
					xmms_remote_jump_to_time(0, i);
					break;
				case KEY_NPAGE: /* FIXME */
					i = 0;
					while (itemtab[*actual][i])
						i++;
					if(i==0) break;
					if (item_visible(itemtab[*actual][i - 1]))
						menu_driver(menutab[*actual], REQ_LAST_ITEM);
					else
						menu_driver(menutab[*actual], REQ_SCR_DPAGE);
					break;
				case KEY_PPAGE:
					if (item_visible(itemtab[*actual][0]))
						menu_driver(menutab[*actual], REQ_FIRST_ITEM);
					else
						menu_driver(menutab[*actual], REQ_SCR_UPAGE);
					break;
				case KEY_HOME:
					menu_driver(menutab[*actual], REQ_FIRST_ITEM);
					break;
				case KEY_END:
					menu_driver(menutab[*actual], REQ_LAST_ITEM);
					break;
				case KEY_IC:
					if(*actual==AREA_PLIST) break;
					menu_driver(menutab[*actual], REQ_TOGGLE_ITEM);
					menu_driver(menutab[*actual], REQ_DOWN_ITEM);
					break;
				case '\t':
					switch(*actual) 
					{
						case AREA_PLIST:
							set_menu_fore(menutab[AREA_PLIST], COLOR_PAIR(PLIST_UNMARK));
							set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_MARK));
							wrefresh(wintab[AREA_PLIST]);
							wrefresh(wintab[AREA_FILE]);
							*actual = AREA_FILE;
							break;
						case AREA_DIR:
							set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_UNMARK));
							set_menu_fore(menutab[AREA_PLIST], COLOR_PAIR(PLIST_MARK));
							wrefresh(wintab[AREA_DIR]);
							wrefresh(wintab[AREA_PLIST]);
							*actual = AREA_PLIST;
							break;
						case AREA_FILE:
							set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_UNMARK));
							set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_MARK));
							wrefresh(wintab[AREA_FILE]);
							wrefresh(wintab[AREA_DIR]);
							*actual = AREA_DIR;
							break;
					}
					break;  
				case 27:
					character=wgetch(wintab[*actual]);
					switch(character)
					{
						case '1':
							if(*actual==AREA_DIR) break;
							if(*actual==AREA_PLIST) {
								set_menu_fore(menutab[AREA_PLIST], COLOR_PAIR(PLIST_UNMARK));
								wrefresh(wintab[WIN_PLIST]);
							} 
							if(*actual==AREA_FILE) {
								set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_UNMARK));
								wrefresh(wintab[WIN_FILE]);
							}
							set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_MARK));
							wrefresh(wintab[WIN_DIR]);
							*actual = AREA_DIR;
							break;
						case '2':
							if(*actual==AREA_PLIST) break;
							if(*actual==AREA_DIR) {
								set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_UNMARK));
								wrefresh(wintab[WIN_DIR]);
							}
							if(*actual==AREA_FILE) {
								set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_UNMARK));
								wrefresh(wintab[WIN_FILE]);
							}
							set_menu_fore(menutab[AREA_PLIST], COLOR_PAIR(PLIST_MARK));
							wrefresh(wintab[WIN_PLIST]);
							*actual = AREA_PLIST;
							break;
						case '3':
							if(*actual==AREA_FILE) break;
							if(*actual==AREA_DIR) {
								set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_UNMARK));
								wrefresh(wintab[WIN_DIR]);
							}
							if(*actual==AREA_PLIST) {
								set_menu_fore(menutab[AREA_PLIST], COLOR_PAIR(PLIST_UNMARK));
								wrefresh(wintab[WIN_PLIST]);
							}
							set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_MARK));
							wrefresh(wintab[WIN_FILE]);
							*actual = AREA_FILE;
							break;
					}
					break; 
				case 'R':
					build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST],
							plist_data, menu_width, menu_height, *actual);
					wrefresh(wintab[WIN_PLIST]);
					wprintw(wintab[WIN_STAT],"\n..playlist reloaded");
					wrefresh(wintab[WIN_STAT]);
					break;
				case 'a':
					if (*actual == AREA_PLIST)
						break;
					if (*actual == AREA_DIR) {
						wprintw(wintab[WIN_STAT], "\nThera are gone be directories added. Are You sure? (y/n) ");
						wrefresh(wintab[WIN_STAT]);
						echo();
						i = wgetch(wintab[WIN_STAT]);
						noecho();
						if (i != 'y' && i != 't') {
							wprintw(wintab[WIN_STAT], "\nAborting..");
							wrefresh(wintab[WIN_STAT]);
							break;
						}
					}
					idx = item_index(current_item(menutab[*actual]));
					if(*actual == AREA_DIR)
						add_to_playlist(dir_data->path, dir_data->full_dirnames[idx]);
					else
						add_to_playlist(dir_data->path, dir_data->full_filenames[idx]);

					build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], 
							plist_data, menu_width, menu_height, *actual);
					wrefresh(wintab[WIN_PLIST]);
					wprintw(wintab[WIN_STAT],"\n..added");
					wrefresh(wintab[WIN_STAT]);
					break; 
				case 's':
					if (*actual == AREA_PLIST)
						break;
					if (*actual == AREA_DIR) {
						wprintw(wintab[WIN_STAT], "\nThera are gone be directories added. Are You sure? (y/n)");
						wrefresh(wintab[WIN_STAT]);
						echo();
						i = wgetch(wintab[WIN_STAT]);
						noecho();
						if (i != 'y' && i != 't') {
							wprintw(wintab[WIN_STAT], "\nAborting..");
							wrefresh(wintab[WIN_STAT]);
							break;
						}
					}
					for (i = 0; itemtab[*actual][i]; i++) {
						if (item_value(itemtab[*actual][i])) {
							/* i is index */
							if(*actual == AREA_DIR)
								add_to_playlist(dir_data->path, dir_data->full_dirnames[i]);
							else
								add_to_playlist(dir_data->path, dir_data->full_filenames[i]);
							set_item_value(itemtab[*actual][i], 0);
						}
					}
					build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], 
							plist_data, menu_width, menu_height, *actual);
					wrefresh(wintab[WIN_PLIST]);
					wprintw(wintab[WIN_STAT], "\n..added");
					wrefresh(wintab[WIN_STAT]);
					break;
				case '\n':
					if (*actual == AREA_FILE) {
						idx = item_index(current_item(menutab[AREA_FILE]));
						add_to_playlist(dir_data->path, dir_data->full_filenames[idx]);
						build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], 
								plist_data, menu_width, menu_height, *actual);
						wrefresh(wintab[WIN_PLIST]);
					}
					if (*actual == AREA_DIR) {
						idx = item_index(current_item(menutab[AREA_DIR]));
						if (handle_newpath(&dir_data->path, dir_data->full_dirnames[idx]))
							/* return to main function to read new dir */
							return 0;
					}
					if (*actual == AREA_PLIST)
						xmms_remote_set_playlist_pos(0, item_index(current_item(menutab[AREA_PLIST])));
					break;
				case 'h':
					tmp = 20;
					wattrset(wintab[WIN_STAT], COLOR_PAIR(HIGHLIGHT) | A_BOLD );
					wprintw(wintab[WIN_STAT],"\nHELP:");
					wattrset(wintab[WIN_STAT], COLOR_PAIR(NORMAL) );
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"a add active");
					wprintw(wintab[WIN_STAT],"%-*s",  tmp,	"A add selected");
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"C clear playlist");
					wprintw(wintab[WIN_STAT],"%-*s",  tmp,	"l playlist length");
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"i actual title");
					wprintw(wintab[WIN_STAT],"%-*s",  tmp,	"r repeat toggle");
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"s shuffle toggle");
					wprintw(wintab[WIN_STAT],"%-*s",  tmp,	"D remove ...");
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"S clear this window");
					wprintw(wintab[WIN_STAT],"%-*s",  tmp,	"I xmms status");
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"J jump to ...");
					wprintw(wintab[WIN_STAT],"%-*s",  tmp,	"j pattern jump");
					wprintw(wintab[WIN_STAT],"\n%-*s",tmp,	"R refresh playlist");

					wrefresh(wintab[WIN_STAT]);
					break;
				case 'x':
					xmms_remote_play(0);
					break;
				case 'z':
					xmms_remote_playlist_prev(0);
					break;
				case 'c':
					xmms_remote_stop(0);
					break;
				case 'v':
					xmms_remote_pause(0);
					break;
				case 'b':
					xmms_remote_playlist_next(0);
					break;
				case 'C':
					xmms_remote_playlist_clear(0);
					build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], 
							plist_data, menu_width, menu_height, *actual);
					wrefresh(wintab[WIN_PLIST]);
					break;
				case 'l':
					wprintw(wintab[WIN_STAT], "\nPlaylist length: %d", xmms_remote_get_playlist_length(0));
					wrefresh(wintab[WIN_STAT]);
					break;
				case '+':
					xmms_remote_get_volume(0, &vl, &vr);
					xmms_remote_set_volume(0, vl + 1, vr + 1);
					wprintw(wintab[WIN_STAT], "\nVolume %d:%d", vl + 1, vr + 1);
					wrefresh(wintab[WIN_STAT]);
					break;
				case '-':
					xmms_remote_get_volume(0, &vl, &vr);
					if (vl < 1)
						vl = 1;
					if (vr < 1)
						vr = 1;
					xmms_remote_set_volume(0, vl - 1, vr - 1);
					wprintw(wintab[WIN_STAT], "\nVolume %d:%d", vl - 1, vr - 1);
					wrefresh(wintab[WIN_STAT]);
					break;
				case 'i':
					i = xmms_remote_get_playlist_pos(0);
					j = i - menu_height/2;
					if(j<0)
						j=0;
					if(plist_data->size - j < menu_height)
						j=plist_data->size - menu_height;
					set_top_row(menutab[AREA_PLIST],j);
					set_current_item(menutab[AREA_PLIST], itemtab[AREA_PLIST][i]);
					wprintw(wintab[WIN_STAT], "\nTitle: %s", xmms_remote_get_playlist_title(0, xmms_remote_get_playlist_pos(0)));
					wrefresh(wintab[WIN_STAT]);
					break;
				case 'r':
					if (xmms_remote_is_repeat(0))
						wprintw(wintab[WIN_STAT], "\nRepeat is off");
					else
						wprintw(wintab[WIN_STAT], "\nRepeat is on");
					xmms_remote_toggle_repeat(0);
					wrefresh(wintab[WIN_STAT]);
					break;
				case 'e':
					if (xmms_remote_is_shuffle(0))
						wprintw(wintab[WIN_STAT], "\nShuffle is off");
					else
						wprintw(wintab[WIN_STAT], "\nShuffle is on");
					xmms_remote_toggle_shuffle(0);
					wrefresh(wintab[WIN_STAT]);
					break;
				case 'J':
					i = xmms_remote_get_playlist_length(0);
					if (i == 0) {
						wprintw(wintab[WIN_STAT], "\nPlaylist is empty");
						wrefresh(wintab[WIN_STAT]);
						break;
					}
					wprintw(wintab[WIN_STAT], "\nTrack number: ");
					wrefresh(wintab[WIN_STAT]);
					echo();
					character = wscanw(wintab[WIN_STAT], "%d", &j);
					noecho();
					if (character != 1 || j <= 0 || j > i - 1) {
						wprintw(wintab[WIN_STAT], "\nBad number!");
						wrefresh(wintab[WIN_STAT]);
						break;
					}
					xmms_remote_set_playlist_pos(0, j - 1);
					break;
				case 'S':
					werase(wintab[WIN_STAT]);
					wrefresh(wintab[WIN_STAT]);
					break;
				case 'D':
					i = item_index(current_item(menutab[AREA_PLIST]));
					if (i < 0) {
						wprintw(wintab[WIN_STAT], "\nPlaylist is empty");
						wrefresh(wintab[WIN_STAT]);
						break;
					}
					wprintw(wintab[WIN_STAT], "\nNr of entry to delete: ");
					wrefresh(wintab[WIN_STAT]);
					echo();
					character = wscanw(wintab[WIN_STAT], "%d", &j);
					noecho();	
					if (character != 1 || j > xmms_remote_get_playlist_length(0) || j <= 0) {
						wprintw(wintab[WIN_STAT], "\nBad number!");
						wrefresh(wintab[WIN_STAT]);
						break;
					}
					xmms_remote_playlist_delete(0, j - 1);
					build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], 
							plist_data, menu_width, menu_height, *actual);
					if (itemtab[AREA_PLIST][i])
						set_current_item(menutab[AREA_PLIST], itemtab[AREA_PLIST][i]);
					else if (i != 0) {
						if (itemtab[AREA_PLIST][i - 1])
							set_current_item(menutab[AREA_PLIST], itemtab[AREA_PLIST][i - 1]);
					}
					wrefresh(wintab[WIN_PLIST]);
					break;
				case KEY_DC: 
					if (*actual == AREA_PLIST) {
						i = item_index(current_item(menutab[AREA_PLIST]));
						if (i < 0)
							break;

						j = top_row(menutab[AREA_PLIST]);

						xmms_remote_playlist_delete(0, i);
						tmp = build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], 
								plist_data, menu_width, menu_height, *actual); 
						/* Move menu cursor back near deleted entry */ 
						if(tmp>0)
						{
							if(i>tmp-1)
								i=tmp-1;
							if(j>tmp-menu_height)
								j=tmp-menu_height;
							if(j<0)
								j=0;
							set_top_row(menutab[AREA_PLIST],j);
							set_current_item(menutab[AREA_PLIST], itemtab[AREA_PLIST][i]);												
						}
						wrefresh(wintab[AREA_PLIST]);
					}
					break;
				case 'I':
					print_status(wintab[WIN_STAT]);
					break;
				case 'j':
					/* This switches whole keyreading to pattern-read mode
					 * Keys that leave this mode: enter, tab, esc 
					 * This implies activating of playlist window */

					/* Activate playlist window */
					if(*actual!=AREA_PLIST)
					{						
						if(*actual==AREA_DIR) {
							set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_UNMARK));
							wrefresh(wintab[WIN_DIR]);
						}
						if(*actual==AREA_FILE) {
							set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_UNMARK));
							wrefresh(wintab[WIN_FILE]);
						}
						set_menu_fore(menutab[AREA_PLIST], COLOR_PAIR(PLIST_MARK));
						wrefresh(wintab[WIN_PLIST]);
						*actual = AREA_PLIST;
					}

					/* Write info about state change */
					wattrset(wintab[WIN_STAT], COLOR_PAIR(HIGHLIGHT) | A_BOLD );
					wprintw(wintab[WIN_STAT], "\n-- Pattern jump mode --");
					wattrset(wintab[WIN_STAT], COLOR_PAIR(NORMAL) );
					wprintw(wintab[WIN_STAT], "\nPattern: ");
					wrefresh(wintab[WIN_STAT]);

					/* Prepare pattern buffor */
					buff[0]='\0';
					idx=0;

					/* Switch mode */
					pattern_jump_mode=true;
					break;
			}
		}
		halfdelay(2);
	}
	return 0;
} /* main_control() */
