/*
 *  ncxmms
 *  Copyright 2003 Mariusz Gniazdowski
 *
 *  NCXmms is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  NCXmms is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <locale.h>
#include <string.h>
#include <unistd.h>
#include <ncurses.h>
#include <menu.h>
#include <xmms/xmmsctrl.h>
#include "defines.h"
#include "playlist.h"
#include "file.h"
#include "keys.h"
#include "util.h"

/*
 * NAME:		main
 * DESCRIPTION: main entry point
 */

int main()
{
	MENU *menutab[3] = { NULL, NULL, NULL };
	ITEM **itemtab[3] = { NULL, NULL, NULL };
	WINDOW *wintab[8];
	struct dir_data_s dir_data = {NULL, NULL, NULL, 0, 0, NULL, NULL, 0, 0 };
	struct plist_data_s plist_data = {NULL, NULL, 0, 0};
	int	i, 
		x_dlist,y_dlist,
		height_dlist, 
		width_dlist,
		x_flist, y_flist,
		height_flist, 
		width_flist, 
		x_plist, y_plist,
		height_plist, 
		width_plist, 
		x_status, y_status,
		height_status,
		width_status,
		actual = 1, 
		stop = 0;

	setlocale(LC_ALL,getenv("LANG"));

	initscr();
	curs_set(0);
	dir_data.path = (char *) malloc(strlen(START_DIR) + 1);
	strcpy(dir_data.path, START_DIR);

	/* windows size calculating
	 * -2 or -4 means space left for menu stuff 
	 * so these sizes are in fact sizes of menu area 
	 * (only width/height_status means real window sizes */
	height_status = 12;
	x_dlist = 0;
	y_dlist = 0;
	height_dlist  = LINES / 3 - 2;
	width_dlist   = COLS / 2 - 2;
	x_flist = 0;
	y_flist = height_dlist + 2;
	height_flist  = LINES - (height_dlist+2) - 2;
	width_flist   = width_dlist;
	x_plist = width_dlist + 2;
	y_plist = 0;
	height_plist  = LINES - height_status - 2;	
	width_plist   = COLS - (width_dlist+2) - 2;
	x_status = x_plist;
	y_status = height_plist + 2;
	width_status  = width_plist + 2; 
	
	/* DIR list window */
	wintab[AREA_DIR] = newwin(height_dlist + 2, width_dlist + 2, y_dlist, x_dlist);
	wintab[WIN_DIR] = derwin(wintab[AREA_DIR], height_dlist, width_dlist, 1, 1);
	/* FILE list window */
	wintab[AREA_FILE] = newwin(height_flist + 2, width_flist + 2, y_flist, x_flist);
	wintab[WIN_FILE] = derwin(wintab[AREA_FILE], height_flist, width_flist, 1, 1);
	/* play list window */
	wintab[AREA_PLIST] = newwin(height_plist + 2, width_plist + 2, y_plist, x_plist);
	wintab[WIN_PLIST] = derwin(wintab[AREA_PLIST], height_plist, width_plist, 1, 1);
	/* STATUS window */
	wintab[AREA_STAT] = newwin(height_status, width_status, y_status, x_status);
	wintab[WIN_STAT] = derwin(wintab[AREA_STAT], height_status-2, width_status-3, 1, 2);

	/* set terminal properties */
	bkgdset(BLANK);
	wbkgdset(wintab[AREA_FILE], BLANK);
	wbkgdset(wintab[AREA_DIR], BLANK);
	wbkgdset(wintab[AREA_PLIST], BLANK);
	wbkgdset(wintab[AREA_STAT], BLANK);
	meta(wintab[AREA_FILE], TRUE);
	meta(wintab[AREA_DIR], TRUE);
	meta(wintab[AREA_PLIST], TRUE);
	meta(wintab[AREA_STAT], TRUE);
	noecho();
	cbreak();
	keypad(stdscr, 1);
	keypad(wintab[AREA_FILE], 1);
	keypad(wintab[AREA_DIR], 1);
	keypad(wintab[AREA_PLIST], 1);
	keypad(wintab[AREA_STAT], 1);

	start_color();

	init_pair(FILE_MARK, COLOR_RED, COLOR_BLUE);
	init_pair(FILE_UNMARK, COLOR_RED, COLOR_BLACK);
	init_pair(DIR_MARK, COLOR_GREEN, COLOR_BLUE);
	init_pair(DIR_UNMARK, COLOR_GREEN, COLOR_BLACK);
	init_pair(PLIST_MARK, COLOR_YELLOW, COLOR_BLUE);
	init_pair(PLIST_UNMARK, COLOR_YELLOW, COLOR_BLACK);
	init_pair(HIGHLIGHT, COLOR_RED, COLOR_BLACK);

	/* draw borders around each window */
	box(wintab[AREA_FILE], 0, 0);
	box(wintab[AREA_DIR], 0, 0);
	box(wintab[AREA_PLIST], 0, 0);
	box(wintab[AREA_STAT], 0, 0);
	wrefresh(wintab[AREA_FILE]);
	wrefresh(wintab[AREA_DIR]);
	wrefresh(wintab[AREA_PLIST]);
	wrefresh(wintab[AREA_STAT]);
	scrollok(wintab[WIN_STAT], 1);

	/* if xmms is runing, build playlist */
	if (xmms_remote_is_running(0))
		build_playlist(&itemtab[AREA_PLIST], &menutab[AREA_PLIST], wintab[AREA_PLIST], wintab[WIN_PLIST], &plist_data, width_plist, height_plist, actual);
	wrefresh(wintab[WIN_PLIST]);
	/*
	 * MAIN LOOP 
	 */
	while (stop == 0) {
		/* reminder: names of the files must be availble during menu lifetime */
		list_new_dir(&dir_data, width_flist-1, width_dlist-1);

		/* create filemenu items */
		itemtab[AREA_FILE] = (ITEM **) calloc(dir_data.num_files + 1, sizeof(ITEM *));
		for (i = 0; i < dir_data.num_files; i++) {
				itemtab[AREA_FILE][i] = new_item(dir_data.abbrev_filenames[i], NULL);
		}
		itemtab[AREA_FILE][dir_data.num_files] = (ITEM *) 0;

		/* create dirmenu items */
		itemtab[AREA_DIR] = (ITEM **) calloc(dir_data.num_dirs + 1, sizeof(ITEM *));
		for (i = 0; i < dir_data.num_dirs; i++) {
				itemtab[AREA_DIR][i] = new_item(dir_data.abbrev_dirnames[i], NULL);
		}
		itemtab[AREA_DIR][dir_data.num_dirs] = (ITEM *) 0;


		/* create filemenu */
		menutab[AREA_FILE] = new_menu(itemtab[AREA_FILE]);
		menu_opts_off(menutab[AREA_FILE], O_ONEVALUE | O_SHOWDESC);
		set_menu_fore(menutab[AREA_FILE], COLOR_PAIR(FILE_UNMARK));
		set_menu_back(menutab[AREA_FILE], COLOR_PAIR(FILE_UNMARK));
		/* set filemenu windows */
		set_menu_win(menutab[AREA_FILE], wintab[AREA_FILE]);
		set_menu_sub(menutab[AREA_FILE], wintab[WIN_FILE]);
		werase(wintab[WIN_FILE]);
		/* format of the filemenu */
		set_menu_format(menutab[AREA_FILE], height_flist, 1);

		/* create dirmenu */
		menutab[AREA_DIR] = new_menu(itemtab[AREA_DIR]);
		menu_opts_off(menutab[AREA_DIR], O_ONEVALUE | O_SHOWDESC);
		set_menu_fore(menutab[AREA_DIR], COLOR_PAIR(DIR_MARK));
		set_menu_back(menutab[AREA_DIR], COLOR_PAIR(DIR_UNMARK));
		/* set dirmenu windows */
		set_menu_win(menutab[AREA_DIR], wintab[AREA_DIR]);
		set_menu_sub(menutab[AREA_DIR], wintab[WIN_DIR]);
		werase(wintab[WIN_DIR]);
		/* format of the dirmenu */
		set_menu_format(menutab[AREA_DIR], height_dlist, 1);

		post_menu(menutab[AREA_FILE]);
		wrefresh(wintab[WIN_FILE]);
		post_menu(menutab[AREA_DIR]);
		wrefresh(wintab[WIN_DIR]);
		if (main_control(&dir_data, wintab, menutab, itemtab, &actual, &plist_data, width_plist, height_plist))
			stop = 1;

		/* path changed -> free remove menus */
		unpost_menu(menutab[AREA_FILE]);
		unpost_menu(menutab[AREA_DIR]);
		free_menu(menutab[AREA_FILE]);
		free_menu(menutab[AREA_DIR]);
		i = 0;
		while (itemtab[AREA_FILE][i]) {
			free_item(itemtab[AREA_FILE][i]);
			i++;
		}
		i = 0;
		while (itemtab[AREA_DIR][i]) {
			free_item(itemtab[AREA_DIR][i]);
			i++;
		}
		free(itemtab[AREA_FILE]);
		free(itemtab[AREA_DIR]);
	}
	unpost_menu(menutab[AREA_PLIST]);
	free_menu(menutab[AREA_PLIST]);
	i = 0;
	while (itemtab[AREA_PLIST][i]) {
		free_item(itemtab[AREA_PLIST][i]);
		i++;
	}
	free(itemtab[AREA_PLIST]);
	if (plist_data.entries != NULL)
		free_2d_tab((void **) plist_data.entries);
	if (plist_data.abbrev_entries != NULL)
		free_2d_tab((void **) plist_data.abbrev_entries);
	if(dir_data.path)
		free(dir_data.path);
	if(dir_data.full_filenames)
		free_2d_tab((void **)dir_data.full_filenames);
	if(dir_data.abbrev_filenames)
		free_2d_tab((void **)dir_data.abbrev_filenames);
	if(dir_data.full_dirnames)
		free_2d_tab((void **)dir_data.full_dirnames);
	if(dir_data.abbrev_dirnames)
		free_2d_tab((void **)dir_data.abbrev_dirnames);

	endwin();

	return 0;
}
