/*
 *  ncxmms
 *  Copyright 2003 Mariusz Gniazdowski
 *
 *  NCXmms is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  NCXmms is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <ncurses.h>
#include <menu.h>
#include <xmms/xmmsctrl.h>
#include "defines.h"
#include "playlist.h"
#include "util.h"

/*
 * NAME:        build_playlist()
 * DESCRIPTION: frees prev playlist, gets newone from xmms and
 * 		builds menu from it
 * 		items 		- its a pointer to table containing pointers to ITEM data
 * 		menu 		- its a pointer to variable containing pointer to MENU data
 * 		main, sub 	- windows that will be associated to menu
 * 		entries 	- pointer to table containing strings keept in menu
 * 				  (i.e. associated to items),
 * 		abbrev_entries	- pointer to table containing strings displaied in menu
 * 				  (i.e. associated to items),
 * 		menu_height	- height of menu,
 * 		abbrev_len	- how long
 * 		actual		- actually active window (to choose selection color)
 */

int build_playlist(ITEM ***items, MENU **menu, WINDOW *main, WINDOW *sub,
		struct plist_data_s *plist_data,
		int menu_width, int menu_height, int actual)
{

	int plist_size = 0, i;
	char *tmp;

	werase(sub);
	/* first free previous menu,items... */
	if (*menu != NULL) {
		unpost_menu(*menu);
		free_menu(*menu);
		*menu = NULL;
	}
	if (*items != NULL) {
		i = 0;
		while ((*items)[i]) {
			free_item((*items)[i]);
			i++;
		}
		free(*items);
		*items = NULL;
	}
	if (plist_data->entries != NULL) {
		free_2d_tab((void **) (plist_data->entries));
		plist_data->entries = NULL;
	}
	if (plist_data->abbrev_entries != NULL) {
		free_2d_tab((void **) (plist_data->abbrev_entries));
		plist_data->abbrev_entries = NULL;
	}

	plist_size = (int) xmms_remote_get_playlist_length(0);
	/* allocate space for items pointers */
	*items = (ITEM **) malloc((plist_size + 1) * sizeof(ITEM *));
	if (*items == NULL)
		return -1;
	/* for free_2d_tab */
	(*items)[plist_size] = NULL;

	/* allocate memory for the playlist entries */
	plist_data->entries = (char **) malloc((plist_size + 1) * sizeof(char *));
	if (plist_data->entries == NULL) {
		free(*items);
		return -1;
	}
	/* for free_2d_tab */
	(plist_data->entries)[plist_size] = NULL;

	/* copy playlist entries to our memory */
	for (i = 0; i < plist_size; i++) {
		tmp = (char *) xmms_remote_get_playlist_title(0, i);
		(plist_data->entries)[i] = (char *) malloc(sizeof(char) * (strlen(tmp) + 1));
		strcpy((plist_data->entries)[i], tmp);
	}

	/* build coresponding abbrev table
	 * only this data is used by menu,
	 * as it has proper length and no
	 * illegal chars (non printable) */
	build_abbrev(plist_data->entries, &plist_data->abbrev_entries, plist_size, menu_width-1);

	/* build items using abbrev_entries table data */
	for (i = 0; i < plist_size; i++) 
		(*items)[i] = new_item((plist_data->abbrev_entries)[i], NULL);

	*menu = new_menu(*items);
	menu_opts_off(*menu, O_ONEVALUE | O_SHOWDESC);
	if(actual==2)
		set_menu_fore(*menu, COLOR_PAIR(PLIST_MARK));
	else
		set_menu_fore(*menu, COLOR_PAIR(PLIST_UNMARK));
	set_menu_back(*menu, COLOR_PAIR(PLIST_UNMARK));
	/* set playlistmenu windows */
	set_menu_win(*menu, main);
	set_menu_sub(*menu, sub);
	/* format of the playlistmenu */
	set_menu_format(*menu, menu_height, 1);
	post_menu(*menu); /* XXX */
	plist_data->size=plist_size;
	return plist_size;
}

/*
 * NAME:        add_to_playlist
 * DESCRIPTION: 
 */

void add_to_playlist(const char *path, const char *name)
{
	char *fullpath;
	fullpath = (char *) calloc(strlen(path) + strlen(name) + 2, sizeof(char));
	sprintf(fullpath, "%s/%s", path, name);
	xmms_remote_playlist_add_url_string(0, (gchar *) fullpath);
	free(fullpath);
	return;
}
