/*
 *  ncxmms
 *  Copyright 2003 Mariusz Gniazdowski
 *
 *  NCXmms is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  NCXmms is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "util.h"

/*
 * NAME:        strcmp_wrap
 * DESCRIPTION: this is a wrapper func for qsort
 */

int strcmp_wrap(const void *a, const void *b)
{
	return strcasecmp(*((const char **) a), *((const char **) b));
}

/*
 * NAME:        free_2d_tab
 * DESCRIPTION: frees memory for rows, then memory for pointers for rows
 * 				last row pointer must be NULL
 */

void free_2d_tab(void **tab)
{
	int i = 0;

	while (tab[i]) {
		free(tab[i]);
		i++;
	}
	free(tab);
}

/*
 * NAME:		build_abbrev
 * DESCRIPTION: makes table of abbrevs from given char** 
 */

int build_abbrev(char **src, char ***dst, int src_size, int abbrev_len)
{
	int i,n;

	*dst = (char **) malloc( (src_size+1)*sizeof(char *) );
	if(NULL == *dst)
		return 0;

	(*dst)[src_size] = NULL;

	for(i=0; i<src_size; i++) {
		n=strlen(src[i]);
		if(n > abbrev_len) 
			n = abbrev_len;
		(*dst)[i] = (char *) malloc( (n+1) * sizeof(char) ); // XXX no checking is done here..
		strncpy((*dst)[i], src[i], n);
		(*dst)[i][n] = '\0';		
		// Changes !isprint-able characters to '_'
		fix_nonprintable( &(*dst)[i] );
	}
	return 1;
}

/* NAME:	fix_nonprintable(char **txt)
 * DESCRIPTION:	Changes chars from other than getenv("LANG") encoding
 * 		to '_' because menu cant handle them.
 */

void fix_nonprintable(char **txt)
{
	int i=0;
	if(!(*txt))
		return;
	if(strlen(*txt)==0) {
		free(*txt);
		*txt = malloc(strlen("(empty name)") + 1);
		strcpy(*txt,"(empty name)");
		return;
	}
	while((*txt)[i]) {
		if(!isprint((*txt)[i]))
			(*txt)[i]='_';
		i++;
	}
}
